@@ -1,3 +1,5 @@
+import copy
+import warnings
 from abc import ABC
 from abc import abstractmethod
 from typing import Optional
@@ -24,7 +26,7 @@ class Feature(ABC):
         self.register_hedger(None)
 
     @abstractmethod
-    def __getitem__(self, time_step: Optional[int]) -> Tensor:
+    def get(self, time_step: Optional[int]) -> Tensor:
         """Return feature tensor.
 
         Returned tensor should have a shape :math:`(N, 1)` where
@@ -54,9 +56,10 @@ class Feature(ABC):
         Returns:
             self
         """
-        self.register_derivative(derivative)
-        self.register_hedger(hedger)
-        return self
+        output = copy.copy(self)
+        output.register_derivative(derivative)
+        output.register_hedger(hedger)
+        return output
 
     def register_derivative(self, derivative: Derivative) -> None:
         setattr(self, "derivative", derivative)
@@ -71,6 +74,11 @@ class Feature(ABC):
         # If a feature uses the state of a hedger, it is state dependent.
         return getattr(self, "hedger") is not None
 
+    # TODO(simaki) Remove later
+    def __getitem__(self, time_step: Optional[int]) -> Tensor:
+        # raise DeprecationWarning("Use `<feature>.get(time_step)` instead")
+        return self.get(time_step)
+
 
 class StateIndependentFeature(Feature):
     # Features that does not use the state of the hedger.
@@ -78,7 +86,9 @@ class StateIndependentFeature(Feature):
     derivative: Derivative
     hedger: None
 
-    def of(self: T, derivative: Derivative, hedger: Optional[Module] = None) -> T:
-        self.register_derivative(derivative)
-        self.register_hedger(None)
-        return self
+    def of(
+        self: "StateIndependentFeature",
+        derivative: Derivative,
+        hedger: Optional[Module] = None,
+    ) -> "StateIndependentFeature":
+        return super().of(derivative=derivative, hedger=None)
