@@ -14,7 +14,7 @@ from .base import Derivative
 
 
 class EuropeanOption(BaseOption):
-    """A European option.
+    r"""A European option.
 
     A European option provides its holder the right to buy (for call option)
     or sell (for put option) an underlying asset with the strike price
@@ -24,7 +24,7 @@ class EuropeanOption(BaseOption):
 
     .. math::
 
-        \\mathrm{payoff} = \\max(S - K, 0)
+        \mathrm{payoff} = \max(S - K, 0)
 
     Here, :math:`S` is the underlying asset's price at maturity and
     :math:`K` is the strike price (`strike`) of the option.
@@ -33,21 +33,16 @@ class EuropeanOption(BaseOption):
 
     .. math::
 
-        \\mathrm{payoff} = \\max(K - S, 0)
+        \mathrm{payoff} = \max(K - S, 0)
+
+    .. seealso::
+        :func:`pfhedge.nn.functional.european_payoff`: Payoff function.
 
     Args:
         underlier (:class:`Primary`): The underlying instrument of the option.
         call (bool, default=True): Specifies whether the option is call or put.
         strike (float, default=1.0): The strike price of the option.
         maturity (float, default=20/250): The maturity of the option.
-        dtype (torch.dtype, optional): Desired device of returned tensor.
-            Default: If None, uses a global default
-            (see :func:`torch.set_default_tensor_type()`).
-        device (torch.device, optional): Desired device of returned tensor.
-            Default: if None, uses the current device for the default tensor type
-            (see :func:`torch.set_default_tensor_type()`).
-            ``device`` will be the CPU for CPU tensor types and
-            the current CUDA device for CUDA tensor types.
 
     Attributes:
         dtype (torch.dtype): The dtype with which the simulated time-series are
@@ -95,6 +90,28 @@ class EuropeanOption(BaseOption):
         >>> derivative.spot
         tensor([[0.0113, 0.0028, 0.0006, 0.0009, 0.0028, 0.0049],
                 [0.0113, 0.0060, 0.0130, 0.0180, 0.0131, 0.0220]])
+
+        Add a knock-out clause with a barrier at 1.03:
+
+        >>> _ = torch.manual_seed(42)
+        >>> derivative = EuropeanOption(BrownianStock(), maturity=5/250)
+        >>> derivative.simulate(n_paths=8)
+        >>> derivative.payoff()
+        tensor([0.0000, 0.0000, 0.0113, 0.0414, 0.0389, 0.0008, 0.0000, 0.0000])
+        >>>
+        >>> def knockout(derivative, payoff):
+        ...     max = derivative.underlier.spot.max(-1).values
+        ...     return payoff.where(max < 1.03, torch.zeros_like(max))
+        >>>
+        >>> derivative.add_clause("knockout", knockout)
+        >>> derivative
+        EuropeanOption(
+          strike=1., maturity=0.0200
+          clauses=['knockout']
+          (underlier): BrownianStock(sigma=0.2000, dt=0.0040)
+        )
+        >>> derivative.payoff()
+        tensor([0.0000, 0.0000, 0.0113, 0.0000, 0.0000, 0.0008, 0.0000, 0.0000])
     """
 
     def __init__(
@@ -112,7 +129,13 @@ class EuropeanOption(BaseOption):
         self.strike = strike
         self.maturity = maturity
 
-        self.to(dtype=dtype, device=device)
+        # TODO(simaki): Remove later. Deprecated for > v0.12.3
+        if dtype is not None or device is not None:
+            self.to(dtype=dtype, device=device)
+            raise DeprecationWarning(
+                "Specifying device and dtype when constructing a Derivative is deprecated."
+                "Specify them in the constructor of the underlier instead."
+            )
 
     def extra_repr(self):
         params = []
@@ -122,7 +145,7 @@ class EuropeanOption(BaseOption):
         params.append("maturity=" + _format_float(self.maturity))
         return ", ".join(params)
 
-    def payoff(self) -> Tensor:
+    def payoff_fn(self) -> Tensor:
         return european_payoff(self.ul().spot, call=self.call, strike=self.strike)
 
 
