@@ -10,23 +10,21 @@ from torch import Tensor
 from pfhedge._utils.doc import _set_attr_and_docstring
 from pfhedge._utils.doc import _set_docstring
 from pfhedge._utils.str import _format_float
+from pfhedge._utils.typing import TensorOrScalar
 from pfhedge.stochastic import generate_geometric_brownian
 
 from .base import Primary
 
-TensorOrFloat = Union[Tensor, float]
-
 
 class BrownianStock(Primary):
-    """A stock of which spot prices follow the geometric Brownian motion.
+    r"""A stock of which spot prices follow the geometric Brownian motion.
 
-    The drift of the spot prices is assumed to be vanishing.
-
-    See :func:`pfhedge.stochastic.generate_geometric_brownian`
-    for details of the process.
+    .. seealso::
+        - :func:`pfhedge.stochastic.generate_geometric_brownian`:
+          The stochastic process.
 
     Args:
-        sigma (float, default=0.2): The parameter :math:`\\sigma`,
+        sigma (float, default=0.2): The parameter :math:`\sigma`,
             which stands for the volatility of the spot price.
         cost (float, default=0.0): The transaction cost rate.
         dt (float, default=1/250): The intervals of the time steps.
@@ -47,7 +45,6 @@ class BrownianStock(Primary):
           :math:`T` is the number of time steps.
 
     Examples:
-
         >>> from pfhedge.instruments import BrownianStock
         >>>
         >>> _ = torch.manual_seed(42)
@@ -104,7 +101,7 @@ class BrownianStock(Primary):
         self,
         n_paths: int = 1,
         time_horizon: float = 20 / 250,
-        init_state: Optional[Tuple[TensorOrFloat]] = None,
+        init_state: Optional[Tuple[TensorOrScalar]] = None,
     ) -> None:
         """Simulate the spot price and add it as a buffer named ``spot``.
 
