@@ -1,3 +1,4 @@
+from typing import Callable
 from typing import List
 from typing import Optional
 from typing import Tuple
@@ -17,6 +18,7 @@ from pfhedge._utils.hook import save_prev_output
 from pfhedge._utils.lazy import has_lazy
 from pfhedge._utils.operations import ensemble_mean
 from pfhedge._utils.str import _format_float
+from pfhedge._utils.typing import TensorOrScalar
 from pfhedge.features import FeatureList
 from pfhedge.features._base import Feature
 from pfhedge.instruments.base import Instrument
@@ -27,8 +29,6 @@ from pfhedge.nn.functional import terminal_value
 from .loss import EntropicRiskMeasure
 from .loss import HedgeLoss
 
-TensorOrFloat = Union[Tensor, float]
-
 
 class Hedger(Module):
     """Module to hedge and price derivatives.
@@ -207,7 +207,7 @@ class Hedger(Module):
     def extra_repr(self) -> str:
         return "inputs=" + str(self.inputs)
 
-    def get_input(self, time_step: Optional[int]) -> Tensor:
+    def get_input(self, derivative: Derivative, time_step: Optional[int]) -> Tensor:
         """Returns the input tensor to the model at the given time step.
 
         Note:
@@ -218,6 +218,7 @@ class Hedger(Module):
             before calling this method.
 
         Args:
+            derivative (Derivative): The derivative used for getting the input.
             time_step (int, optional): The time step to get the input tensor.
                 If ``None`` an input tensor for all time steps is returned.
 
@@ -241,10 +242,10 @@ class Hedger(Module):
             >>> derivative.simulate()
             >>> hedger = Hedger(Naked(), ["time_to_maturity", "volatility"])
             >>> _ = hedger.compute_pnl(derivative, n_paths=1)  # Materialize features
-            >>> hedger.get_input(0)
+            >>> hedger.get_input(derivative, 0)
             tensor([[[0.0800, 0.2000]]])
         """
-        return self.inputs[time_step]
+        return self.inputs.of(derivative=derivative).get(time_step)
 
     def compute_hedge(
         self, derivative: Derivative, hedge: Optional[List[Instrument]] = None
@@ -284,7 +285,7 @@ class Hedger(Module):
             tensor([[0.5056, 0.5295, 0.5845, 0.6610, 0.2918, 0.2918],
                     [0.5056, 0.3785, 0.4609, 0.5239, 0.7281, 0.7281]])
         """
-        self.inputs = self.inputs.of(derivative, self)
+        inputs = self.inputs.of(derivative, self)
         hedge = cast(List[Instrument], [derivative.ul()] if hedge is None else hedge)
 
         # Check that the spot prices of the hedges have the same sizes
@@ -292,19 +293,19 @@ class Hedger(Module):
             raise ValueError("The spot prices of the hedges must have the same size")
 
         (n_paths, n_steps), n_hedges = hedge[0].spot.size(), len(hedge)
-        if self.inputs.is_state_dependent():
-            zeros = torch.zeros((n_paths, 1, n_hedges)).to(hedge[0].spot)
+        if inputs.is_state_dependent():
+            zeros = hedge[0].spot.new_zeros((n_paths, 1, n_hedges))
             save_prev_output(self, input=None, output=zeros)
             outputs = []
             for time_step in range(n_steps - 1):
-                input = self.get_input(time_step)  # (N, T=1, F)
+                input = inputs.get(time_step)  # (N, T=1, F)
                 outputs.append(self(input))  # (N, T=1, H)
             outputs.append(outputs[-1])
             output = torch.cat(outputs, dim=-2)  # (N, T, H)
         else:
             # If all features are state-independent, compute the output at all
             # time steps at once, which would be faster.
-            input = self.get_input(None)  # (N, T, F)
+            input = inputs.get(None)  # (N, T, F)
             output = self(input)  # (N, T, H)
             # This maintains consistency with the previous implementations.
             # In previous implementation for loop is computed for 0...T-2 and
@@ -320,7 +321,7 @@ class Hedger(Module):
         derivative: Derivative,
         hedge: Optional[List[Instrument]] = None,
         n_paths: int = 1000,
-        init_state: Optional[Tuple[TensorOrFloat, ...]] = None,
+        init_state: Optional[Tuple[TensorOrScalar, ...]] = None,
     ) -> Tensor:
         """Returns the terminal portfolio value after hedging a given derivative.
 
@@ -376,7 +377,7 @@ class Hedger(Module):
         hedge: Optional[List[Instrument]] = None,
         n_paths: int = 1000,
         n_times: int = 1,
-        init_state: Optional[Tuple[TensorOrFloat, ...]] = None,
+        init_state: Optional[Tuple[TensorOrScalar, ...]] = None,
         enable_grad: bool = True,
     ) -> Tensor:
         """Returns the value of the criterion for the terminal portfolio value
@@ -430,16 +431,18 @@ class Hedger(Module):
         return mean_loss
 
     def _configure_optimizer(
-        self, derivative: Derivative, optimizer: Union[Optimizer, type]
+        self,
+        derivative: Derivative,
+        optimizer: Union[Optimizer, Callable[..., Optimizer]],
     ) -> Optimizer:
-        if isinstance(optimizer, type):
+        if not isinstance(optimizer, Optimizer):
             if has_lazy(self):
                 # Run a placeholder forward to initialize lazy parameters
                 _ = self.compute_pnl(derivative, n_paths=1)
             # If we use `if issubclass(optimizer, Optimizer)` here, mypy thinks that
             # optimizer is Optimizer rather than its subclass (e.g. Adam)
             # and complains that the required parameter default is missing.
-            if Optimizer in optimizer.__mro__:
+            if Optimizer in getattr(optimizer, "__mro__", []):
                 optimizer = cast(Optimizer, optimizer(self.model.parameters()))
             else:
                 raise TypeError("optimizer is not an Optimizer type")
@@ -452,8 +455,8 @@ class Hedger(Module):
         n_epochs: int = 100,
         n_paths: int = 1000,
         n_times: int = 1,
-        optimizer=Adam,
-        init_state: Optional[Tuple[TensorOrFloat, ...]] = None,
+        optimizer: Union[Optimizer, Callable[..., Optimizer]] = Adam,
+        init_state: Optional[Tuple[TensorOrScalar, ...]] = None,
         verbose: bool = True,
         validation: bool = True,
     ) -> Optional[List[float]]:
@@ -566,7 +569,7 @@ class Hedger(Module):
         hedge: Optional[List[Instrument]] = None,
         n_paths: int = 1000,
         n_times: int = 1,
-        init_state: Optional[Tuple[TensorOrFloat, ...]] = None,
+        init_state: Optional[Tuple[TensorOrScalar, ...]] = None,
         enable_grad: bool = False,
     ) -> Tensor:
         """Evaluate the premium of the given derivative.
